/* -*- tab-width: 4; indent-tabs-mode: nil -*-
 *
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is The JavaScript Debugger.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Robert Ginda, <rginda@netscape.com>, original author
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

body {
    margin: 0px;
    padding: 0px;
}

source-listing {
    display: block;
    width: 100%;
    font-family: monospace, sans-serif;
}

line {
    width: 100%;
    white-space: pre;
    display: block;
}

line[highlighted] {
    background: #EEEEEE;
}

line[highlighted] > num {
    border-right: 2px #919bd6 solid;
    background: #d5d5e0;
}

line[stoppedAt] {
    background: #ecef34;
}

line[stoppedAt] > num {
    font-weight: bold;
    border-right: 2px orange solid;
    background: #ecef34;
}

margin {
    -moz-user-select: none;
    font-weight: bold;
    background: #CCCCCC;
    padding-bottom: 0.5em;
    white-space: pre;
    cursor: pointer;
}

margin[x="t"] {
    color: #444444;
}

margin[x="t"]:hover {
    color: slategrey;
}

margin[f="t"] {
    background: orange;
    color: white;
}

margin[b="t"] {
    background: red;
    color: white;
}

num {
    -moz-user-select: none;
    padding-bottom: 0.5em;
    padding-right: 5px;
    margin-right: 10px;
    border-left: 1px grey solid;
    border-right: 2px #CCCCCC solid;
    background: #EEEEEE;
}

k {
    font-weight: bold;
}

c {
    color: steelblue;
}

t {
    color: darkgreen;
}
